<?php

namespace Tests\Feature;

use App\Models\Especialidade;
use Tests\BrowserKitTest as TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;

class ManageEspecialidadeTest extends TestCase
{
    use RefreshDatabase;

    /** @test */
    public function user_can_see_especialidade_list_in_especialidade_index_page()
    {
        $especialidade = Especialidade::factory()->create();

        $this->loginAsUser();
        $this->visitRoute('especialidades.index');
        $this->see($especialidade->name);
    }

    /** @test */
    public function user_can_create_a_especialidade()
    {
        $this->loginAsUser();
        $this->visitRoute('especialidades.index');

        $this->click(__('especialidade.create'));
        $this->seeRouteIs('especialidades.index', ['action' => 'create']);

        $this->submitForm(__('especialidade.create'), [
            'name'        => 'Especialidade 1 name',
            'description' => 'Especialidade 1 description',
        ]);

        $this->seeRouteIs('especialidades.index');

        $this->seeInDatabase('especialidades', [
            'name'        => 'Especialidade 1 name',
            'description' => 'Especialidade 1 description',
        ]);
    }

    private function getCreateFields(array $overrides = [])
    {
        return array_merge([
            'name'        => 'Especialidade 1 name',
            'description' => 'Especialidade 1 description',
        ], $overrides);
    }

    /** @test */
    public function validate_especialidade_name_is_required()
    {
        $this->loginAsUser();

        // name empty
        $this->post(route('especialidades.store'), $this->getCreateFields(['name' => '']));
        $this->assertSessionHasErrors('name');
    }

    /** @test */
    public function validate_especialidade_name_is_not_more_than_60_characters()
    {
        $this->loginAsUser();

        // name 70 characters
        $this->post(route('especialidades.store'), $this->getCreateFields([
            'name' => str_repeat('Test Title', 7),
        ]));
        $this->assertSessionHasErrors('name');
    }

    /** @test */
    public function validate_especialidade_description_is_not_more_than_255_characters()
    {
        $this->loginAsUser();

        // description 256 characters
        $this->post(route('especialidades.store'), $this->getCreateFields([
            'description' => str_repeat('Long description', 16),
        ]));
        $this->assertSessionHasErrors('description');
    }

    /** @test */
    public function user_can_edit_a_especialidade_within_search_query()
    {
        $this->loginAsUser();
        $especialidade = Especialidade::factory()->create(['name' => 'Testing 123']);

        $this->visitRoute('especialidades.index', ['q' => '123']);
        $this->click('edit-especialidade-'.$especialidade->id);
        $this->seeRouteIs('especialidades.index', ['action' => 'edit', 'id' => $especialidade->id, 'q' => '123']);

        $this->submitForm(__('especialidade.update'), [
            'name'        => 'Especialidade 1 name',
            'description' => 'Especialidade 1 description',
        ]);

        $this->seeRouteIs('especialidades.index', ['q' => '123']);

        $this->seeInDatabase('especialidades', [
            'name'        => 'Especialidade 1 name',
            'description' => 'Especialidade 1 description',
        ]);
    }

    private function getEditFields(array $overrides = [])
    {
        return array_merge([
            'name'        => 'Especialidade 1 name',
            'description' => 'Especialidade 1 description',
        ], $overrides);
    }

    /** @test */
    public function validate_especialidade_name_update_is_required()
    {
        $this->loginAsUser();
        $especialidade = Especialidade::factory()->create(['name' => 'Testing 123']);

        // name empty
        $this->patch(route('especialidades.update', $especialidade), $this->getEditFields(['name' => '']));
        $this->assertSessionHasErrors('name');
    }

    /** @test */
    public function validate_especialidade_name_update_is_not_more_than_60_characters()
    {
        $this->loginAsUser();
        $especialidade = Especialidade::factory()->create(['name' => 'Testing 123']);

        // name 70 characters
        $this->patch(route('especialidades.update', $especialidade), $this->getEditFields([
            'name' => str_repeat('Test Title', 7),
        ]));
        $this->assertSessionHasErrors('name');
    }

    /** @test */
    public function validate_especialidade_description_update_is_not_more_than_255_characters()
    {
        $this->loginAsUser();
        $especialidade = Especialidade::factory()->create(['name' => 'Testing 123']);

        // description 256 characters
        $this->patch(route('especialidades.update', $especialidade), $this->getEditFields([
            'description' => str_repeat('Long description', 16),
        ]));
        $this->assertSessionHasErrors('description');
    }

    /** @test */
    public function user_can_delete_a_especialidade()
    {
        $this->loginAsUser();
        $especialidade = Especialidade::factory()->create();
        Especialidade::factory()->create();

        $this->visitRoute('especialidades.index', ['action' => 'edit', 'id' => $especialidade->id]);
        $this->click('del-especialidade-'.$especialidade->id);
        $this->seeRouteIs('especialidades.index', ['action' => 'delete', 'id' => $especialidade->id]);

        $this->seeInDatabase('especialidades', [
            'id' => $especialidade->id,
        ]);

        $this->press(__('app.delete_confirm_button'));

        $this->dontSeeInDatabase('especialidades', [
            'id' => $especialidade->id,
        ]);
    }
}
